//////////////////////////////////////////////////////////////////////
//this header contains the structs/typedefs that the user might access
//////////////////////////////////////////////////////////////////////
#ifndef audioxtreme_h_included
#define audioxtreme_h_included

#include <windows.h>
#include <dmusicc.h>
#include <dmusici.h>

#define MAX_ALLEG_VOICE     64


#define MOD_STOPPED 1
#define MOD_PAUSED 2
#define MOD_PLAYING 3
#define MIDI_STOPPED 1
#define MIDI_PAUSED 2
#define MIDI_PLAYING 3

//typedefs for module and sample
typedef struct ENVELOPE_INFO
{
    int env[12];
    int pos[12];
    int flg;
    int pts;
    int loopbeg;
    int loopend;
    int susbeg;
    int susend;
    int a;
    int b;
    int p;
    int v;

}ENVELOPE_INFO;


typedef struct CHANNEL_INFO
{
    ENVELOPE_INFO volenv;
    ENVELOPE_INFO panenv;

    int instrument;
    int sample;
    int volume;
    int note;
    int period;
    int c2spd;
    int transpose;
    int pan;
    int kick;       // TRUE if sample needs to be restarted
    int keyon;
    int volfade;   // volume fadeout
    int instfade;  // how much volume to subtract from volfade

    int temp_volume;
    int temp_period;
    int temp_pan;

    int pan_slide_common;
    int pan_slide;
    int pan_slide_left;
    int pan_slide_right;

    int pro_pitch_slide_on;
    int pro_pitch_slide;
    int pro_fine_pitch_slide;
    int s3m_pitch_slide_on;
    int s3m_pitch_slide;
    int s3m_fine_pitch_slide;
    int xm_pitch_slide_up_on;
    int xm_pitch_slide_up;
    int xm_pitch_slide_down_on;
    int xm_pitch_slide_down;
    int xm_fine_pitch_slide_up;
    int xm_fine_pitch_slide_down;
    int xm_extra_fine_pitch_slide_up;
    int xm_x_up;
    int xm_x_down;

    int pro_volume_slide;
    int s3m_volume_slide_on;
    int s3m_fine_volume_slide;
    int s3m_volume_slide;
    int xm_volume_slide_on;
    int xm_volume_slide;
    int xm_fine_volume_slide_up;
    int xm_fine_volume_slide_down;

    int loop_on;
    int loop_times;
    int loop_start;

    int tremolo_on;
    int tremolo_waveform;
    char tremolo_pointer;
    int tremolo_speed;
    int tremolo_depth;
    int tremolo_shift;
    
    int vibrato_on;    
    int vibrato_waveform;
    char vibrato_pointer;
    int vibrato_speed;
    int vibrato_depth;
    int vibrato_shift;

    int slide2period_on;
    int slide2period_spd;
    int slide2period;

    int arpeggio_on;
    int arpeggio;

    int tremor_on;
    int tremor_count;
    int tremor_set;

    int delay_sample;
    int cut_sample;
    int glissando;
    int retrig;
    int s3m_retrig_on;
    int s3m_retrig;
    int s3m_retrig_slide;

    int sample_offset_on;
    int sample_offset;

    int global_volume_slide_on;
    int global_volume_slide;

}CHANNEL_INFO; 

typedef struct MUSIC_INFO
{
    int max_chn;
    int no_chn;

    int tick;
    int pos;
    int pat;
    int trk;
    int flag;

    int bpm;
    int tempo;
    int speed_ratio;
    int pitch_ratio;
    int global_volume;
    
    int new_pos;        // for pattern break
    int new_trk;        // or position jump
    int pattern_delay;  // pattern delay

    int skip_pos;       // for next_pattern
    int skip_trk;       // or prev_pattern
    int loop;           // replay the music if ended
    int pause;          // for pause function
    int forbid;
    int is_playing;

}MUSIC_INFO; 

typedef struct NOTE_INFO
{
    int sample;
    int note;
    int volume;
    int command;
    int extcommand;
}NOTE_INFO;


typedef struct SAMPLE_INFO
{
    int lenght;
    int c2spd;
    int transpose;
    int volume;
    int pan;
    int repoff;
    int replen;
    int loop;

    int vibrato_type;
    int vibrato_spd;
    int vibrato_depth;
    int vibrato_rate;

}SAMPLE_INFO;


typedef struct INSTRUMENT_INFO
{
    int sample_number[96];

    int volenv[12];
    int volpos[12];
    int no_volenv;
    int vol_type;
    int vol_susbeg;
    int vol_susend;
    int vol_begin;
    int vol_end;

    int panenv[12];
    int panpos[12];
    int no_panenv;
    int pan_type;
    int pan_susbeg;
    int pan_susend;
    int pan_begin;
    int pan_end;

    int volume_fadeout;

}INSTRUMENT_INFO;


typedef struct PATTERN_INFO
{
    NOTE_INFO *ni;
    int no_pos;
}PATTERN_INFO;


typedef struct AX_MIDI_TAG {
	IDirectMusicSegment* pMIDISeg;
} AX_MIDI;


typedef struct SAMPLE_TAG                 /* a sample */
{
   int bits;                           /* 8 or 16 */
   int stereo;                         /* sample type flag */
   int freq;                           /* sample frequency */
   int priority;                       /* 0-255 */
   int volume;
   int pan;
   //int finetune;
   unsigned long len;                  /* length (in samples) */
   unsigned long loop_start;           /* loop start position */
   unsigned long loop_end;             /* loop finish position */
   unsigned long param;                /* for internal use by the driver */
   void *data;                         /* sample data */
} SAMPLE;

typedef struct AX_MODULE_TAG
{
    char name[29];
    SAMPLE_INFO *si;
    PATTERN_INFO *pi;
    INSTRUMENT_INFO *ii;
    SAMPLE *s;

    int no_trk;
    int no_pat;
    int pat_table[256];
    int panning[MAX_ALLEG_VOICE];
    int flag;

    int tempo;
    int bpm;

    int restart_pos;       
    int no_chn;
    int no_instrument;
    int no_sample;
    int global_volume;

}AX_MODULE;


/////////////////////////////////////////////////////////////////////////////////////////////////////////

//system funcs.
__declspec(dllimport) bool _export CALLBACK InstallSound(int voices);
__declspec(dllimport) void _export CALLBACK RemoveSound(void);
__declspec(dllimport) bool CALLBACK MusicIsPlaying(void);

//sample funcs.
__declspec(dllimport) SAMPLE* CALLBACK LoadSample(char* filename);
__declspec(dllimport) void CALLBACK UnloadSample(SAMPLE* sample);
__declspec(dllimport) int CALLBACK PlaySample(SAMPLE* sample,bool repeat=false);
__declspec(dllimport) void CALLBACK StopSample(int voice);
__declspec(dllimport) void CALLBACK AdjustSample(SAMPLE* spl,int volume=-1,int pan=-1,int frequency=-1);
												  


//midi music functions (dmusic)
__declspec(dllimport) bool CALLBACK InitMidi(void);
__declspec(dllimport) void CALLBACK KillMidi(void);
__declspec(dllimport) AX_MIDI* CALLBACK LoadMidi(char* filename);
__declspec(dllimport) void CALLBACK UnloadMusic(AX_MIDI* midi);
__declspec(dllimport) void CALLBACK PlayMusic(AX_MIDI* music,int times_to_play=1);
__declspec(dllimport) void CALLBACK StopMusic(void);


//module functions
__declspec(dllimport) AX_MODULE* CALLBACK LoadMod(char* filename);
__declspec(dllimport) void CALLBACK UnloadMusic(AX_MODULE *);
__declspec(dllimport) void CALLBACK PlayMusic(AX_MODULE *,int times_to_repeat=1);
__declspec(dllimport) char* CALLBACK ModName(AX_MODULE*);


#endif
